'
'----------------------     Fuer Pollin Display 16080 mit Touch      ------------------------
'--------------------------------------------------------------------------------------------
'
'
' Alex D. (allu)   9/2009
'
' Hinweis:
'         - Wenn Fehler zu ADCSRA oder ADCSR gemeldet werden, den jeweils
'         .....anderen Registernamen einsetzen.
'
' Angepasst an Mega32, 16/11/2009
'           , , , a, , ,  in den Charaktergenerator eingebaut,
'           .....gem ASCII-Zeichensatz, deutsche Variante.
'
' Hinweis zum Mega32:
'         - JTAG-Interface des Mega32 muss ausgeschaltet sein,
'         .....ansonsten knnen PC2 bis PC4 nicht verwendet werden
'
' Hinweis zum UART und PD0 und PD1
'         - Falls die Baudrate fr die serielle Schnittstelle initialisiert wurde,
'         ..... . sind PD0 und PD1 nicht mehr verfgbar
'
' Gendert: 17/11/2009
'           - Freie Port und Bit Wahl fr den Control-Bus zum Display eingebaut.
'           - Hardware-Anpassungen in einem Block zusammemgefasst
'           - Display-Ansteuerungs-Zeiten bis 20 Mhz-Taktfrequenz angepasst
'
' Gendert: 18/11/2009
'           - Ohm- und C-Symbol (Grad Celsius) ergnzt.
'
' Gendert: 20/11/2009
'           - Programmteile ausgalagert,
'           ..... aufgeteilt in RomTabs.txt, LCD16080.txt, Touch16080.txt
'           ..... Initialisierung
'
' Ergnzt:  27/12/2009
'           - Klammergesteuerte automatische Umrahmung fr (Taste) mit dnnen Rahmen
'           ...  und mit dicker Rahmen [xyz]

'           - Automatische Umcodierung von Zeichen zur einfachen Eingabe ber Tastaur
'           .... _ erzeugt Symbol ZURCK,            # erzeugt Symbol ENTER,
'           .....< erzeugt Symbol PFEIL KLEINER,     > erzeugt Symbol PFEIL GRSSER,
'           .....$ erzeugt Symbol PFEIL NACH UNTEN,  & erzeugt Symbol PFEIL NACH OBEN,
'
'---------------------------  Information zu einzelnen Funktionen ---------------------------
'--------------------------------------------------------------------------------------------
'
' Locate_zeile = 31                               Schreibe Text auf Zeile 31
' Locate_spalte = 0                               Schreibe Text ab Spalte 0
' Muster    = "01234567890123456789012"           Muster dient als Voralge fr die Gestaltung
' Lcd_string ="Die ist ein Testext    "           Darstellung eines Textes auf dem Display
' Gosub Lcd_write_string                          Textstring verarbeiten und ausgeben
'
'
' Muster     = "01234567890123456789012"          Muster dient als Vorlage fr die Gestaltung
' Lcd_form_3 = "(7     ) Az 7_ (4     )"          Die Umklammerung erzeugt eine Funktionstaste = (128+Zeichencode)
'                                                 Positionen 0 bis 7 = Funktionstaste, Wert:(128 + "7")
'                                                 Postion 8 = Leer, kein Rckgabewert zugewiesen = 0
'                                                 Postion 9 = ASCII-Code "A"
'                                                 Postion 10 = ASCII-Code "z"
'                                                 Postion 11 = Leer, kein Rckgabewert zugewiesen = 0
'                                                 Postion 12 = ASCII-Code "7"
'                                                 Postion 13 = "_" wird in Spacezeichen " " umgewndelt
'                                                 Postion 14 = Leer, kein Rckgabewert zugewiesen = 0
'                                                 Positionen 15 bis 22 = Funktionstaste, Wert:(128 + "4")
'                                                 Fr alle Space wird eine 0 zurckgegeben
'                                  .....
' Gosub Lcd_form_aufbau                           Generiert daraus alle Funtionstasten
'
'
'
' weiter mit Beschreibungen.....
'
'------------------------------------------------------------------
'Konfiguration der Mikrocontrollerumgebung
'------------------------------------------------------------------

$regfile = "m32def.DAT"                                     ' ATMega32
$crystal = 16000000                                         ' 16MHz
' Config Com1 = 19200 , Synchrone = 0 , Parity = None , Stopbits = 1 , Databits = 9 , Clockpol = 0
                     ' Reserviert TxD und RxD fr die serielle Schnittstelle
                     ' PD0 und PD1 sind dann nicht mehr verfgbar

$hwstack = 128
$swstack = 256
$framesize = 128

'------------------------------------------------------------------
'------------------------------------------------------------------
'PortPins zum Display 16080 festlegen und initialisieren
'------------------------------------------------------------------
'------------------------------------------------------------------
'------------------------------------------------------------------
' Daten-Bus zum Display (hier auf PortC)
'------------------------------------------------------------------
Daten_bus_out Alias Portc
Daten_bus_in Alias Pinc
Daten_ddr Alias Ddrc

'------------------------------------------------------------------
' Steuersignale zum Display (hier auf PortD)
'------------------------------------------------------------------
Control_bus_rs Alias Portd
Control_bus_rs_ddr Alias Ddrd
Rs Alias 6                                                  'RS Pin Display

Control_bus_rw Alias Portd
Control_bus_rw_ddr Alias Ddrd
Rw Alias 5                                                  'R/W Pin Display

Control_bus_e Alias Portd
Control_bus_e_ddr Alias Ddrd
E Alias 7                                                   'Enable Pin Display

Control_bus_cs Alias Portd
Control_bus_cs_ddr Alias Ddrd
Cs Alias 4                                                  'Chipsel. Pin Display

Control_bus_r Alias Portd
Control_bus_r_ddr Alias Ddrd
R Alias 3

'------------------------------------------------------------------
' AD-Wandler Touchanschluss
' ADC0 = Y- / ADC1 = Y+  / ADC2 = X+ / ADC3 = X-
'------------------------------------------------------------------
Touch_control Alias Porta
Touch_ddr Alias Ddra

'------------------------------------------------------------------
' ENDE Portanpassung
'------------------------------------------------------------------

'------------------------------------------------------------------
'      Daten-Bus und Controlbus zum Display initialisieren
'------------------------------------------------------------------

Gosub Display_ports_initialisieren

' und weiter mit dem Initialisieren
$include "Initialisierung_1.txt"

Dim Irq_teiler As Byte                                      ' Vorteiler fr Touchaufruf
Dim Ticker As Byte                                          ' Vorteiler fr die Timer
Dim Timer_a As Byte                                         ' Timer A = 0,02 bis 5,1 sec
Dim Timer_b As Byte                                         ' Timer B = 0,02 bis 5,1 sec

'------------------------------------------------------------------
'------------------------------------------------------------------
'Verstndigung zwischen Bascom und Touch
'------------------------------------------------------------------
'------------------------------------------------------------------
                                                            ' Touch meldet:
Dim Touch_meldet As Byte                                    ' Berhrung erkannt = ja = 1
Ja Alias 1                                                  ' ... 1 = Touchberhrung wurde erkannt
Erledigt Alias 0                                            ' ... 0-setzen fr Quittung = Bearbeitet

Dim Touch_aktiviert As Byte                                 ' Solange der Touch berhrt wird = 2, sonst 0

Dim X_aktuell As Byte                                       ' X-Feinwert 0 bis 255, Berhrungspunkt                                                   '
Dim Y_aktuell As Byte                                       ' Y-Feinwert 0 bis 255, Berhrungspunkt

Dim X_melden As Byte                                        ' X-Rasterwert 0 bis 22, Klalibriert
Dim Y_melden As Byte                                        ' Y-Rasterwert 0 bis 7, Klalibriert

Dim F_taste As Byte                                         ' Rckgabewert des berhrten Rasterskstchens

'------------------------------------------------------------------
'------------------------------------------------------------------
'Verstndigung zwischen Bascom und Display
'------------------------------------------------------------------
'------------------------------------------------------------------

' Der Inhalt von LCD-String wird mit Gosub LCD_Write ab der
' ..... Position Locate_Spalte und Locate_Zeile ins Display geschrieben
' ..... Achtung: keine berlaufbremse eingebaut, kein Zeilenumbruch
_
Dim Locate_zeile As Byte                                    ' 0 bis 79 = 80 Zeilen
Dim Locate_spalte As Byte                                   ' 0 bis 22 = 23 Spalten
Dim Lcd_string As String * 23                               ' wird ins Displays geschrieben

'------------------------------------------------------------------
' Vorbereitet fr Texteingabe ber Touch
'------------------------------------------------------------------
Dim Touch_string As String * 23
Dim Touch_string_zeiger As Byte



'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------

Gosub Lcd_clear

Locate_zeile = 05                                           ' Schreiben kann man ab jeder Zeile 0 bis 79
Locate_spalte = 0                                           ' eine Zeichenzeile ist 8 Zeilen hoch
Lcd_string = "  P I X E L   T E S T  "
Gosub Lcd_write_string

Dim Command As Byte
Dim Datenwert As Byte


'-- 1. Pixel ---
Command = &H0A                                              'Befehl: RAM-Zeiger Low
Datenwert = &H00                                            'Spalte 1 (a 8 Pixel 0-22)
Gosub Zum_glcd                                              'In Display-RAM schreiben

Command = &H0B                                              'Befehl: RAM-Zeiger High
Datenwert = &H00
Gosub Zum_glcd                                              'In Display-RAM schreiben

Command = &H0F                                              'Befehl: Pixel setzen
Datenwert = &H01                                            'Pixel 1 (1-8)
Gosub Zum_glcd                                              'In Display-RAM schreiben

'-- 2. Pixel ---
'Command = &H0A                                              'Befehl: RAM-Zeiger Low
'Datenwert = &H01                                            'Spalte 2 (a 8 Pixel 0-22)
'Gosub Zum_glcd                                              'In Display-RAM schreiben

'Command = &H0B                                              'Befehl: RAM-Zeiger High
'Datenwert = &H00
'Gosub Zum_glcd                                              'In Display-RAM schreiben

'Command = &H0F                                              'Befehl: Pixel setzen
'Datenwert = &H01                                            'Pixel 1 (1-8)
'Gosub Zum_glcd


'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
'-[TEST-PIXEL SETZEN]-----------------------------------------------------------------------
Do
nop
Loop





'------------------------------------------------------------------
' Tabelle zur Aufnahme der Rckgabewerte im Raster
' Gosub Lcd_form_clear lscht die Rckgabewerte
' Gosub Lcd_form_aufbau erzeugt die Funktiontasten
'------------------------------------------------------------------
Dim Lcd_format(192) As Byte
Dim Lcd_form_0 As String * 23 At Lcd_format(1) Overlay
Dim Lcd_form_1 As String * 23 At Lcd_format(25) Overlay
Dim Lcd_form_2 As String * 23 At Lcd_format(49) Overlay
Dim Lcd_form_3 As String * 23 At Lcd_format(73) Overlay
Dim Lcd_form_4 As String * 23 At Lcd_format(97) Overlay
Dim Lcd_form_5 As String * 23 At Lcd_format(121) Overlay
Dim Lcd_form_6 As String * 23 At Lcd_format(145) Overlay
Dim Lcd_form_7 As String * 23 At Lcd_format(169) Overlay

 '------------------------------------------------------------------
' Hier werden die Kalibrierwerte dauerhaft aufgehoben
'------------------------------------------------------------------
$eeprom
   Touch_kal_werte:                                         ' Platzhalter und Defaultwerte

   Data Kalibriert                                          ' schon kalibriert ?
   Data 49                                                  ' X_1_Pos = rechter Randabstand
   Data 65                                                  ' Y_1_Pos = linker Randabstand
   Data 122                                                 ' X_pro_Spaltenbreite
   Data 90                                                  ' Y_pro_Zeilenhhe
$data

Gosub Display_und_touch_initialisieren

'------------------------------------------------------------------
' Hier beginnt das Hauptprogramm
'------------------------------------------------------------------
Dim Temp_a As Word
Dim Temp_b As Integer
Dim Temp_c As Byte

'------------------------------------------------------------------
' Funktionstasten definieren
'------------------------------------------------------------------
_kalibrieren Alias 1 + 48 + 128
_texteingabe Alias 2 + 48 + 128
_delete Alias 3 + 48 + 128
_touch_werte_anzeigen Alias 4 + 48 + 128
_menue Alias 5 + 48 + 128
_symbole_anzeigen Alias 6 + 48 + 128
_zeichensatz_anzeigen Alias 7 + 48 + 128

'------------------------------------------------------------------
'------------------------------------------------------------------
' Hauptmenue
'------------------------------------------------------------------
'------------------------------------------------------------------

Gosub Auswahl







Do
   If Touch_meldet = Ja Then
      Touch_meldet = Erledigt

      Select Case F_taste
         Case _kalibrieren : Gosub Touch_kalibrieren
         Case _texteingabe : Gosub Zeicheneingabe
         Case _touch_werte_anzeigen : Gosub Touch_werte_anzeigen
         Case _symbole_anzeigen : Gosub Symbole_anzeigen
         Case _zeichensatz_anzeigen : Gosub Zeichensatz_anzeigen
      End Select

   End If
Loop

'------------------------------------------------------------------
'------------------------------------------------------------------
' Auswahlmen
'------------------------------------------------------------------
'------------------------------------------------------------------
Auswahl:

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 07                                           ' Schreiben kann man ab jeder Zeile 0 bis 79
Locate_spalte = 0                                           ' eine Zeichenzeile ist 8 Zeilen hoch
'Muster    = "01234567890123456789012"                      ' Das Tastenraster ist aber fest , 23 * 8 Tasten
Lcd_string = "    A U S W A H L"
Gosub Lcd_write_string


Locate_zeile = 26
Locate_spalte = 0
'Muster    = "01234567890123456789012"                      ' Spaltenanzeiger 0 bis 22 = 23 Zeichen
Lcd_string = "(Chr-Liste )(Touchtest)"                      ' Klammern in eine Umrahmung bersetzen
Lcd_form_2 = "(7         )(4        )"                      ' Den Funktionstasten Werte zuweisen
Lcd_form_3 = "(7         )(4        )"                      ' ...fr eine 2 Zeilen hohe Flche
Gosub Lcd_taste_malen                                       ' Klammern in eine Umrahmung bersetzen ... oder
'Gosub Lcd_write_string                                      ' Text anzeigen so wie er ist, keine Umrandung


Locate_zeile = 46
Locate_spalte = 0
'Muster    = "01234567890123456789012"                      ' Spaltenanzeiger 0 bis 22 = 23 Zeichen
Lcd_string = "  (Symbole anzeigen)"                         ' Klammern in eine Umrahmung bersetzen
Lcd_form_4 = "  (6               )"                         ' Den Funktionstasten Werte zuweisen
Lcd_form_5 = "  (6               )"                         ' ...fr eine 2 Zeilen hohe Flche
Gosub Lcd_taste_malen                                       ' Klammern in eine Umrahmung bersetzen ... oder
'Gosub Lcd_write_string                                      ' Text anzeigen so wie er ist, keine Umrandung


Locate_zeile = 66
Locate_spalte = 0
'Muster    = "01234567890123456789012"                      ' Spaltenanzeiger 0 bis 22 = 23 Zeichen
Lcd_string = " ( Kal.)      ( Text )"                       ' Klammern in eine Umrahmung bersetzen
Lcd_form_6 = " (1    )      (2     )"                       ' Den Funktionstasten Werte zuweisen
Lcd_form_7 = " (1    )      (2     )"                       ' ...fr eine 2 Zeilen hohe Flche
Gosub Lcd_taste_malen                                       ' Klammern in eine Umrahmung bersetzen ... oder
'Gosub Lcd_write_string                                      ' Text anzeigen so wie er ist, keine Umrandung

Gosub Lcd_form_aufbau                                       'Funktionstasten generieren fr Schreibfaule

Return


'------------------------------------------------------------------
'------------------------------------------------------------------
' Tasten mit Symbolen anzeigen
'------------------------------------------------------------------
'------------------------------------------------------------------

Symbole_anzeigen:

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 66
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "              (#)  [_]"                       ' Symbol Return und zurck
Lcd_form_6 = "              RRR  (5)"
Lcd_form_7 = "              RRR  (5)"
Gosub Lcd_taste_malen

Locate_zeile = 36
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = " (<) (>)"                                     'links und  rechts
Lcd_form_3 = " lll rrr"
Lcd_form_4 = " lll rrr"
Gosub Lcd_taste_malen

Locate_zeile = 16
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "   (&)"
Lcd_form_1 = "   ooo"
Lcd_form_2 = "   ooo"                                       '   nach oben
Gosub Lcd_taste_malen

Locate_zeile = 56
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "   ($)"
Lcd_form_5 = "   uuu"
Lcd_form_6 = "   uuu"                                       '   nach unten
Gosub Lcd_taste_malen

Gosub Lcd_form_aufbau

Do
If Touch_meldet = Ja Then

Locate_zeile = 26
Locate_spalte = 10
Lcd_string = "            "
  Select Case F_taste
      Case "l" : Lcd_string = "nach links  "
      Case "r" : Lcd_string = "nach rechts "
      Case "o" : Lcd_string = "nach oben   "
      Case "u" : Lcd_string = "nach unten  "
      Case "R" : Lcd_string = "Returnsymbol"
   End Select
Gosub Lcd_write_string

Touch_meldet = Erledigt

End If

Loop Until F_taste = _menue
Gosub Auswahl

Return



























'------------------------------------------------------------------
'------------------------------------------------------------------
' Zeichenvorrat anzeigen:
'------------------------------------------------------------------
'------------------------------------------------------------------

Zeichensatz_anzeigen:

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 36
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "                   [_]"
Lcd_form_3 = "                   (5)"
Lcd_form_4 = "                   (5)"

'Gosub Lcd_write_string
Gosub Lcd_taste_malen

Gosub Lcd_form_aufbau

Temp_c = 0

For Temp_b = 0 To 70 Step 10
   Locate_zeile = Temp_b
   Locate_spalte = 1

   For Temp_a = 0 To 15
      Lcd_string = Chr(temp_c)
      Incr Temp_c
      Gosub Lcd_write_string
   Next Temp_a

Next Temp_b

Do
Loop Until F_taste = _menue
Gosub Auswahl

Return



'------------------------------------------------------------------
'------------------------------------------------------------------
' Zum Testen der Touchfunktion
'------------------------------------------------------------------
'------------------------------------------------------------------

Touch_werte_anzeigen:

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 01
Locate_spalte = 0
Lcd_string = "01234567890123456789012"
Lcd_form_0 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 11
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "1                     "
Lcd_form_1 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 21
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "2                     "
Lcd_form_2 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 31
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "3                     "
Lcd_form_3 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 41
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "4                     "
Lcd_form_4 = "4                     "
Gosub Lcd_write_string


'------------------------------------------------------------------
' Zurcktaste in die beiden Zeilen 51 und 61 eintragen > 51 + 5 =56
'------------------------------------------------------------------

Locate_zeile = 51
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "5                 (5) "                      ' Funktionstaste "5" ab Poas 18 = Zurck
Lcd_form_5 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 61
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "6                 (5) "                      ' Funktionstaste "5" ab Pos 18 = Zurck
Lcd_form_6 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 56                                           ' Zurcktaste ab Pos 18 auf einkleben, dafr 51 +5
Locate_spalte = 18                                          ' drei Zeichen breit und 2 Zeichenzeilen hoch
Lcd_string = "[_]"
Gosub Lcd_taste_malen

'------------------------------------------------------------------
'   Fertisch! Zurcktaste ist breit umrandet
'------------------------------------------------------------------

Locate_zeile = 71
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "7abcdefghijklmnopqrstu"
Lcd_form_7 = Lcd_string
Gosub Lcd_write_string

Gosub Lcd_form_aufbau

'------------------------------------------------------------------
' Kalibrierwerte anzeigen
'------------------------------------------------------------------

Locate_zeile = 51
Locate_spalte = 2
Lcd_string = "X1P=" + Str(x_1_pos) + "  Y1P=" + Str(y_1_pos)
Gosub Lcd_write_string

Locate_zeile = 61
Locate_spalte = 2
Lcd_string = "XpS=" + Str(x_pro_spaltenbreite) + " YpZ=" + Str(y_pro_zeilenhoehe)
Gosub Lcd_write_string



Do

'------------------------------------------------------------------
' Touch Berhrung anzeigen
'------------------------------------------------------------------

      Locate_zeile = 25
      Locate_spalte = 19
      If Touch_aktiviert > 1 Then
         Lcd_string = Chr(7)
      Else
         Lcd_string = Chr(6)
      End If
      Gosub Lcd_write_string


   If Touch_meldet = Ja Then

      Locate_zeile = 21
      Locate_spalte = 2
      Lcd_string = "Raster:Y=" + Str(y_melden) + " X=" + Str(x_melden) + "    "
      Gosub Lcd_write_string

      Locate_zeile = 31
      Locate_spalte = 2
      Lcd_string = "Fein:Y=" + Str(y_aktuell) + " X=" + Str(x_aktuell) + "    "
      Gosub Lcd_write_string

      Lcd_string = "Chr:" + Chr(f_taste) + " Wert=" + Str(f_taste) + "    "
      Locate_zeile = 11
      Locate_spalte = 2
      Gosub Lcd_write_string

      'Muster    = "234567890123456789012"
      Lcd_string = "Rckgabewert:" + Hex(f_taste) + "h"
      Locate_zeile = 41
      Locate_spalte = 2
      Gosub Lcd_write_string

      Touch_meldet = Erledigt
   End If

Loop Until F_taste = _menue
Gosub Auswahl
Return


'------------------------------------------------------------------
'------------------------------------------------------------------
' Test Zeicheneingabe
'------------------------------------------------------------------
'------------------------------------------------------------------

Zeicheneingabe:

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 21
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = " ABCDEFGHIJKLMNMOP 123 "
Lcd_form_2 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 31
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = " QRSTUVWXYZ     456"
Lcd_form_3 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 41
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = " abcdefghijklmnop  789"
Lcd_form_4 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 51
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = " qrstuvwxyz    0 _"
Lcd_form_5 = Lcd_string
Gosub Lcd_write_string


Locate_zeile = 66
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "              (DEL) [_]"                      'Erzeugt eine Zurcktaste , [Dick] umrandet
Lcd_form_6 = "              (3  ) (5)"                      'DEL soll die Lschtaste sein , (Fein) umrandet
Lcd_form_7 = "              (3  ) (5)"
Gosub Lcd_taste_malen

Locate_zeile = 61
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = " Input:"
Gosub Lcd_write_string


Gosub Lcd_form_aufbau


Touch_string = " "
Touch_string_zeiger = 1

Locate_zeile = 71
Locate_spalte = 0
Lcd_string = Touch_string + "-"
Gosub Lcd_write_string



Do
If Touch_meldet = Ja Then

   If F_taste < 128 Then
      Lcd_string = "Zeichen :" + Chr(f_taste) + " Wert=" + Hex(f_taste) + "h"
   Else
      Lcd_string = "Funktion:" + Chr(f_taste) + " Wert=" + Hex(f_taste) + "h"
   End If
   Locate_zeile = 11
   Locate_spalte = 0
   Gosub Lcd_write_string


   Locate_zeile = 01
   Locate_spalte = 0
   Lcd_string = "Position:  Y=" + Str(y_melden) + "  X=" + Str(x_melden) + "   "
   Gosub Lcd_write_string


'------------------------------------------------------------------
' Eingabetext aufbauen
'------------------------------------------------------------------
   If F_taste > 1 And F_taste < 128 And Touch_string_zeiger < 12 Then
         Touch_string = Touch_string + Chr(f_taste)
         Incr Touch_string_zeiger
         Locate_zeile = 71
         Locate_spalte = 0
         If Touch_string_zeiger > 12 Then
            Lcd_string = Touch_string
         Else
         Lcd_string = Touch_string + "-"
         End If
         Gosub Lcd_write_string
   End If

   If F_taste = _delete Then
      If Touch_string_zeiger > 1 Then
            Decr Touch_string_zeiger
            Touch_string = Left(touch_string , Touch_string_zeiger)
            Locate_zeile = 71
            Locate_spalte = 0
            Lcd_string = Touch_string + "- "
            Gosub Lcd_write_string
      End If
    End If


   Touch_meldet = Erledigt

End If

Loop Until F_taste = _menue
Gosub Auswahl
Return



'------------------------------------------------------------------
'------------------------------------------------------------------
'Unterprogramme
'------------------------------------------------------------------
'------------------------------------------------------------------

'------------------------------------------------------------------
' Display und Touch initialisieren
' Display und Touch initialisieren
' GLcd im Grafikmode mit 23 Spalten initialisieren:
'------------------------------------------------------------------

Display_und_touch_initialisieren:
   Waitms 50

   Gosub Reset_an_display_senden
   Waitms 10

   Gosub Lcd_initialisieren
   Gosub Lcd_clear
   Gosub Touch_init

'------------------------------------------------------------------
'------------------------------------------------------------------
'
'  Timer -interupt Initialisieren - > Touch , Time_out , Datenbertragung , Uhr
'  Ergebnis =(prozessortakt / 400) -1   ergibt 20 ms IRQ-Abstand
'------------------------------------------------------------------
'------------------------------------------------------------------

$asm
Irq_int_timer1:
      ldi   Temp_1, &H4e                                    'High-Byte vom Ergebnis (OCR1AH, ocr1al)
      !out ocr1ah, Temp_1

      ldi   Temp_1, &H1f                                    'Low-Byte vom Ergebnis (ocr1ah, OCR1AL)
      !out ocr1al, Temp_1

       ldi   Temp_1, &B0000_1010                            'Vorteiler auf geteilt/64, Mode CTC
      !out tccr1b, Temp_1

      ldi   Temp_1, &B0001_0000                             'IRQ fr CTC freigeben
      !out timsk, Temp_1
$end Asm


:
On Oc1a Irq_timer
Enable Interrupts                                           'interrupts freigeben

'------------------------------------------------------------------
'------------------------------------------------------------------
'  Hur fr Test:   Gosub Touch_default_werte_eintragen
'   Betrieb:       Gosub Touch_kal_werte_laden
'------------------------------------------------------------------

'   Gosub Touch_default_werte_eintragen
   Gosub Touch_kal_werte_laden
   Return
'------------------------------------------------------------------
'------------------------------------------------------------------




'------------------------------------------------------------------
' Kalibrierwerte aus Eprom laden, falls es welche gibt
'------------------------------------------------------------------

Touch_kal_werte_laden:
         Readeeprom Temp_a , Touch_kal_werte
   If Temp_a = Kalibriert Then
         Readeeprom X_1_pos
         Readeeprom Y_1_pos
         Readeeprom X_pro_spaltenbreite
         Readeeprom Y_pro_zeilenhoehe
   Else
         Gosub Touch_kalibrieren
   End If
Return


'------------------------------------------------------------------
' Touch kalibrieren
'------------------------------------------------------------------

Touch_kalibrieren:

Gosub Lcd_clear
Gosub Lcd_form_clear
Gosub Zielscheiben_malen
   If X_pro_spaltenbreite < 114 Or X_pro_spaltenbreite > 130 Then
         Gosub Touch_default_werte_eintragen
   Elseif Y_pro_zeilenhoehe < 82 Or Y_pro_zeilenhoehe > 98 Then
         Gosub Touch_default_werte_eintragen
   Elseif X_1_pos > 70 Or Y_1_pos > 85 Then
         Gosub Touch_default_werte_eintragen
   Else
         Temp_a = Kalibriert
         Writeeeprom Temp_a , Touch_kal_werte
         Writeeeprom X_1_pos
         Writeeeprom Y_1_pos
         Writeeeprom X_pro_spaltenbreite
         Writeeeprom Y_pro_zeilenhoehe
   End If
   Gosub Auswahl
Return


'------------------------------------------------------------------

Touch_default_werte_eintragen:

   Locate_zeile = 51
   Locate_spalte = 1
   Lcd_string = "KX=" + Str(x_pro_spaltenbreite) + " "
   Gosub Lcd_write_string

   Lcd_string = "KY=" + Str(y_pro_zeilenhoehe)
   Gosub Lcd_write_string
   Lcd_string = " Fehler"
   Gosub Lcd_write_string

   X_pro_spaltenbreite = 122
   Y_pro_zeilenhoehe = 90
   X_1_pos = 49
   Y_1_pos = 65

   Wait 5

Return

'------------------------------------------------------------------
' Zielscheiben malen:
'------------------------------------------------------------------

Zielscheiben_malen:

   'Muster    = "567890123456789012"
   Lcd_string = "Zum Kalibrieren"
   Locate_zeile = 31
   Locate_spalte = 5
   Gosub Lcd_write_string

   'Muster    = "567890123456789012"
   Lcd_string = "Kreuze antippen"
   Locate_zeile = 45
   Locate_spalte = 5
   Gosub Lcd_write_string

   Wait 1

   Lcd_string = Chr(1) + Chr(2)
   Locate_zeile = 06
   Locate_spalte = 0
   Gosub Touch_beruehrung_abwarten
   X_oben_links = X_aktuell
   Y_oben_links = Y_aktuell

   Locate_zeile = 06
   Locate_spalte = 21
   Gosub Touch_beruehrung_abwarten
   X_oben_rechts = X_aktuell
   Y_oben_rechts = Y_aktuell

   Locate_zeile = 66
   Locate_spalte = 0
   Gosub Touch_beruehrung_abwarten
   X_unten_links = X_aktuell
   Y_unten_links = Y_aktuell

   Locate_zeile = 66
   Locate_spalte = 21
   Gosub Touch_beruehrung_abwarten
   X_unten_rechts = X_aktuell
   Y_unten_rechts = Y_aktuell

   Temp_a = X_oben_links + X_unten_links
   Temp_a = Temp_a + 1
   Temp_a = Temp_a / 2
   X_1_pos = Temp_a

   Temp_a = Y_oben_rechts + Y_oben_links
   Temp_a = Temp_a + 1
   Temp_a = Temp_a / 2
   Y_1_pos = Temp_a

   Temp_a = X_oben_rechts - X_oben_links
   Temp_b = X_unten_rechts - X_unten_links
   Temp_a = Temp_a + Temp_b
   Temp_a = 43130 / Temp_a                                  '43008= 2*1024 *21 =rechnerisch, ohne runden
   X_pro_spaltenbreite = Temp_a

   Temp_a = Y_unten_rechts - Y_oben_rechts
   Temp_b = Y_unten_links - X_oben_links
   Temp_a = Temp_a + Temp_b
   Temp_a = 26000 / Temp_a                                  '24567= 2*2048 *6  =rechnerisch, ohne runden
   Y_pro_zeilenhoehe = Temp_a
Return


'------------------------------------------------------------------
Touch_beruehrung_abwarten:
'------------------------------------------------------------------
   Gosub Lcd_write_string                                   'Kreuzchen malen

   Do
   Loop Until Touch_meldet = Ja
   Touch_meldet = Erledigt

   Gosub Lcd_clear
Return



'-------------------------- HINWEIS -------------------------------
' Timer bedienen, Beispiel: Timer_a auf 800msec aufziehen
' Timer_A = 40 ' 40 * 20mse-Zeitbasis
' If Timer_a = 0 then xxxxxx   ' Zeit abgelaufen : Timer_a = 0
'
'------------------------------------------------------------------

Irq_timer:

 Incr Irq_teiler
 If Irq_teiler = 50 Then
    Irq_teiler = 0
          '    Gosub Uhr_ticker   'Sekundentakt fr Uhr
 End If

 Incr Ticker
 If Timer_a > 0 Then Decr Timer_a                           ' Zwei Timer zur freien Verwendung
 If Timer_b > 0 Then Decr Timer_b

 Gosub Touch_auswerten

Return


'------------------------------------------------------------------
' und hier gehts in Assembler weiter ....
'------------------------------------------------------------------

$include "Touch16080_1.txt"
$include "LCD16080_1.txt"
$include "LCD_Tasten_malen.txt"

End

$include "RomTab_1.txt"